@@ -7,7 +7,6 @@
 
 module Command.DropUnused where
 
-import Logs.Unused
 import Common.Annex
 import Command
 import qualified Annex
@@ -15,11 +14,12 @@ import qualified Command.Drop
 import qualified Remote
 import qualified Git
 import qualified Option
+import Command.Unused (withUnusedMaps, UnusedMaps(..), startUnused)
 
 def :: [Command]
 def = [withOptions [Command.Drop.fromOption] $
 	command "dropunused" (paramRepeating paramNumRange)
-		seek "drop unused file content"]
+		seek SectionMaintenance "drop unused file content"]
 
 seek :: [CommandSeek]
 seek = [withUnusedMaps start]
@@ -28,14 +28,14 @@ start :: UnusedMaps -> Int -> CommandStart
 start = startUnused "dropunused" perform (performOther gitAnnexBadLocation) (performOther gitAnnexTmpLocation)
 
 perform :: Key -> CommandPerform
-perform key = maybe droplocal dropremote =<< Remote.byName =<< from
-	where
-		dropremote r = do
-			showAction $ "from " ++ Remote.name r
-			ok <- Remote.removeKey r key
-			next $ Command.Drop.cleanupRemote key r ok
-		droplocal = Command.Drop.performLocal key (Just 0) -- force drop
-		from = Annex.getField $ Option.name Command.Drop.fromOption
+perform key = maybe droplocal dropremote =<< Remote.byNameWithUUID =<< from
+  where
+	dropremote r = do
+		showAction $ "from " ++ Remote.name r
+		ok <- Remote.removeKey r key
+		next $ Command.Drop.cleanupRemote key r ok
+	droplocal = Command.Drop.performLocal key (Just 0) Nothing -- force drop
+	from = Annex.getField $ Option.name Command.Drop.fromOption
 
 performOther :: (Key -> Git.Repo -> FilePath) -> Key -> CommandPerform
 performOther filespec key = do
