@@ -15,6 +15,7 @@ import Data.BloomFilter
 import Data.BloomFilter.Easy
 import Data.BloomFilter.Hash
 import Control.Monad.ST
+import qualified Data.Map as M
 
 import Common.Annex
 import Command
@@ -22,7 +23,6 @@ import Logs.Unused
 import Annex.Content
 import Utility.FileMode
 import Logs.Location
-import Config
 import qualified Annex
 import qualified Git
 import qualified Git.Command
@@ -38,7 +38,7 @@ import Types.Key
 
 def :: [Command]
 def = [withOptions [fromOption] $ command "unused" paramNothing seek
-	"look for unused file content"]
+	SectionMaintenance "look for unused file content"]
 
 fromOption :: Option
 fromOption = Option.field ['f'] "from" paramRemote "remote to check for unused content"
@@ -64,27 +64,26 @@ checkUnused = chain 0
 	, check "bad" staleBadMsg $ staleKeysPrune gitAnnexBadDir
 	, check "tmp" staleTmpMsg $ staleKeysPrune gitAnnexTmpDir
 	]
-	where
-		findunused True = do
-			showNote "fast mode enabled; only finding stale files"
-			return []
-		findunused False = do
-			showAction "checking for unused data"
-			excludeReferenced =<< getKeysPresent
-		chain _ [] = next $ return True
-		chain v (a:as) = do
-			v' <- a v
-			chain v' as
+  where
+	findunused True = do
+		showNote "fast mode enabled; only finding stale files"
+		return []
+	findunused False = do
+		showAction "checking for unused data"
+		excludeReferenced =<< getKeysPresent
+	chain _ [] = next $ return True
+	chain v (a:as) = do
+		v' <- a v
+		chain v' as
 
 checkRemoteUnused :: String -> CommandPerform
-checkRemoteUnused name = go =<< fromJust <$> Remote.byName (Just name)
-	where
-		go r = do
-			showAction "checking for unused data"
-			_ <- check "" (remoteUnusedMsg r) (remoteunused r) 0
-			next $ return True
-		remoteunused r =
-			excludeReferenced <=< loggedKeysFor $ Remote.uuid r
+checkRemoteUnused name = go =<< fromJust <$> Remote.byNameWithUUID (Just name)
+  where
+	go r = do
+		showAction "checking for unused data"
+		_ <- check "" (remoteUnusedMsg r) (remoteunused r) 0
+		next $ return True
+	remoteunused r = excludeReferenced <=< loggedKeysFor $ Remote.uuid r
 
 check :: FilePath -> ([(Int, Key)] -> String) -> Annex [Key] -> Int -> Annex Int
 check file msg a c = do
@@ -100,9 +99,9 @@ number n (x:xs) = (n+1, x) : number (n+1) xs
 
 table :: [(Int, Key)] -> [String]
 table l = "  NUMBER  KEY" : map cols l
-	where
-		cols (n,k) = "  " ++ pad 6 (show n) ++ "  " ++ key2file k
-		pad n s = s ++ replicate (n - length s) ' '
+  where
+	cols (n,k) = "  " ++ pad 6 (show n) ++ "  " ++ key2file k
+	pad n s = s ++ replicate (n - length s) ' '
 
 staleTmpMsg :: [(Int, Key)] -> String
 staleTmpMsg t = unlines $ 
@@ -129,8 +128,8 @@ remoteUnusedMsg :: Remote -> [(Int, Key)] -> String
 remoteUnusedMsg r u = unusedMsg' u
 	["Some annexed data on " ++ name ++ " is not used by any files:"]
 	[dropMsg $ Just r]
-	where
-		name = Remote.name r 
+  where
+	name = Remote.name r 
 
 dropMsg :: Maybe Remote -> String
 dropMsg Nothing = dropMsg' ""
@@ -159,11 +158,11 @@ dropMsg' s = "\nTo remove unwanted data: git-annex dropunused" ++ s ++ " NUMBER\
  -}
 excludeReferenced :: [Key] -> Annex [Key]
 excludeReferenced ks = runfilter firstlevel ks >>= runfilter secondlevel
-	where
-		runfilter _ [] = return [] -- optimisation
-		runfilter a l = bloomFilter show l <$> genBloomFilter show a
-		firstlevel = withKeysReferencedM
-		secondlevel = withKeysReferencedInGit
+  where
+	runfilter _ [] = return [] -- optimisation
+	runfilter a l = bloomFilter show l <$> genBloomFilter show a
+	firstlevel = withKeysReferencedM
+	secondlevel = withKeysReferencedInGit
 
 {- Finds items in the first, smaller list, that are not
  - present in the second, larger list.
@@ -174,19 +173,17 @@ excludeReferenced ks = runfilter firstlevel ks >>= runfilter secondlevel
 exclude :: Ord a => [a] -> [a] -> [a]
 exclude [] _ = [] -- optimisation
 exclude smaller larger = S.toList $ remove larger $ S.fromList smaller
-	where
-		remove a b = foldl (flip S.delete) b a
+  where
+	remove a b = foldl (flip S.delete) b a
 
 {- A bloom filter capable of holding half a million keys with a
  - false positive rate of 1 in 1000 uses around 8 mb of memory,
  - so will easily fit on even my lowest memory systems.
  -}
 bloomCapacity :: Annex Int
-bloomCapacity = fromMaybe 500000 . readish
-	<$> getConfig (annexConfig "bloomcapacity") ""
+bloomCapacity = fromMaybe 500000 . annexBloomCapacity <$> Annex.getGitConfig
 bloomAccuracy :: Annex Int
-bloomAccuracy = fromMaybe 1000 . readish
-	<$> getConfig (annexConfig "bloomaccuracy") ""
+bloomAccuracy = fromMaybe 1000 . annexBloomAccuracy <$> Annex.getGitConfig
 bloomBitsHashes :: Annex (Int, Int)
 bloomBitsHashes = do
 	capacity <- bloomCapacity
@@ -208,8 +205,8 @@ genBloomFilter convert populate = do
 	bloom <- lift $ newMB (cheapHashes numhashes) numbits
 	_ <- populate $ \v -> lift $ insertMB bloom (convert v)
 	lift $ unsafeFreezeMB bloom
-	where
-		lift = liftIO . stToIO
+  where
+	lift = liftIO . stToIO
 
 bloomFilter :: Hashable t => (v -> t) -> [v] -> Bloom t -> [v]
 bloomFilter convert l bloom = filter (\k -> convert k `notElemB` bloom) l
@@ -217,70 +214,75 @@ bloomFilter convert l bloom = filter (\k -> convert k `notElemB` bloom) l
 {- Given an initial value, folds it with each key referenced by
  - symlinks in the git repo. -}
 withKeysReferenced :: v -> (Key -> v -> v) -> Annex v
-withKeysReferenced initial a = withKeysReferenced' initial folda
-	where
-		folda k v = return $ a k v
+withKeysReferenced initial a = withKeysReferenced' Nothing initial folda
+  where
+	folda k _ v = return $ a k v
 
 {- Runs an action on each referenced key in the git repo. -}
 withKeysReferencedM :: (Key -> Annex ()) -> Annex ()
-withKeysReferencedM a = withKeysReferenced' () calla
-	where
-		calla k _ = a k
+withKeysReferencedM a = withKeysReferenced' Nothing () calla
+  where
+	calla k _ _ = a k
 
-withKeysReferenced' :: v -> (Key -> v -> Annex v) -> Annex v
-withKeysReferenced' initial a = do
+{- Folds an action over keys and files referenced in a particular directory. -}
+withKeysFilesReferencedIn :: FilePath -> v -> (Key -> FilePath -> v -> Annex v) -> Annex v
+withKeysFilesReferencedIn = withKeysReferenced' . Just
+
+withKeysReferenced' :: Maybe FilePath -> v -> (Key -> FilePath -> v -> Annex v) -> Annex v
+withKeysReferenced' mdir initial a = do
 	(files, clean) <- getfiles
 	r <- go initial files
 	liftIO $ void clean
 	return r
-	where
-		getfiles = ifM isBareRepo
+  where
+	getfiles = case mdir of
+		Nothing -> ifM isBareRepo
 			( return ([], return True)
 			, do
 				top <- fromRepo Git.repoPath
 				inRepo $ LsFiles.inRepo [top]
 			)
-		go v [] = return v
-		go v (f:fs) = do
-			x <- Backend.lookupFile f
-			case x of
-				Nothing -> go v fs
-				Just (k, _) -> do
-					!v' <- a k v
-					go v' fs
-
+		Just dir -> inRepo $ LsFiles.inRepo [dir]
+	go v [] = return v
+	go v (f:fs) = do
+		x <- Backend.lookupFile f
+		case x of
+			Nothing -> go v fs
+			Just (k, _) -> do
+				!v' <- a k f v
+				go v' fs
 
 withKeysReferencedInGit :: (Key -> Annex ()) -> Annex ()
 withKeysReferencedInGit a = do
 	rs <- relevantrefs <$> showref
 	forM_ rs (withKeysReferencedInGitRef a)
-	where
-		showref = inRepo $ Git.Command.pipeReadStrict [Param "show-ref"]
-		relevantrefs = map (Git.Ref .  snd) .
-			nubBy uniqref .
-			filter ourbranches .
-			map (separate (== ' ')) . lines
-		uniqref (x, _) (y, _) = x == y
-		ourbranchend = '/' : show Annex.Branch.name
-		ourbranches (_, b) = not (ourbranchend `isSuffixOf` b)
-			&& not ("refs/synced/" `isPrefixOf` b)
+  where
+	showref = inRepo $ Git.Command.pipeReadStrict [Param "show-ref"]
+	relevantrefs = map (Git.Ref .  snd) .
+		nubBy uniqref .
+		filter ourbranches .
+		map (separate (== ' ')) . lines
+	uniqref (x, _) (y, _) = x == y
+	ourbranchend = '/' : show Annex.Branch.name
+	ourbranches (_, b) = not (ourbranchend `isSuffixOf` b)
+		&& not ("refs/synced/" `isPrefixOf` b)
 
 withKeysReferencedInGitRef :: (Key -> Annex ()) -> Git.Ref -> Annex ()
 withKeysReferencedInGitRef a ref = do
 	showAction $ "checking " ++ Git.Ref.describe ref
 	go <=< inRepo $ LsTree.lsTree ref
-	where
-		go [] = noop
-		go (l:ls)
-			| isSymLink (LsTree.mode l) = do
-				content <- encodeW8 . L.unpack
-					<$> catFile ref (LsTree.file l)
-				case fileKey (takeFileName content) of
-					Nothing -> go ls
-					Just k -> do
-						a k
-						go ls
-			| otherwise = go ls
+  where
+	go [] = noop
+	go (l:ls)
+		| isSymLink (LsTree.mode l) = do
+			content <- encodeW8 . L.unpack
+				<$> catFile ref (LsTree.file l)
+			case fileKey (takeFileName content) of
+				Nothing -> go ls
+				Just k -> do
+					a k
+					go ls
+		| otherwise = go ls
 
 {- Looks in the specified directory for bad/tmp keys, and returns a list
  - of those that might still have value, or might be stale and removable.
@@ -310,3 +312,49 @@ staleKeys dirspec = do
 			return $ mapMaybe (fileKey . takeFileName) files
 		, return []
 		)
+
+data UnusedMaps = UnusedMaps
+	{ unusedMap :: UnusedMap
+	, unusedBadMap :: UnusedMap
+	, unusedTmpMap :: UnusedMap
+	}
+
+{- Read unused logs once, and pass the maps to each start action. -}
+withUnusedMaps :: (UnusedMaps -> Int -> CommandStart) -> CommandSeek
+withUnusedMaps a params = do
+	unused <- readUnusedLog ""
+	unusedbad <- readUnusedLog "bad"
+	unusedtmp <- readUnusedLog "tmp"
+	return $ map (a $ UnusedMaps unused unusedbad unusedtmp) $
+		concatMap unusedSpec params
+
+unusedSpec :: String -> [Int]
+unusedSpec spec
+	| "-" `isInfixOf` spec = range $ separate (== '-') spec
+	| otherwise = maybe badspec (: []) (readish spec)
+  where
+	range (a, b) = case (readish a, readish b) of
+		(Just x, Just y) -> [x..y]
+		_ -> badspec
+	badspec = error $ "Expected number or range, not \"" ++ spec ++ "\""
+
+{- Start action for unused content. Finds the number in the maps, and
+ - calls either of 3 actions, depending on the type of unused file. -}
+startUnused :: String
+	-> (Key -> CommandPerform)
+	-> (Key -> CommandPerform) 
+	-> (Key -> CommandPerform)
+	-> UnusedMaps -> Int -> CommandStart
+startUnused message unused badunused tmpunused maps n = search
+	[ (unusedMap maps, unused)
+	, (unusedBadMap maps, badunused)
+	, (unusedTmpMap maps, tmpunused)
+	]
+  where
+	search [] = error $ show n ++ " not valid (run git annex unused for list)"
+	search ((m, a):rest) =
+		case M.lookup n m of
+			Nothing -> search rest
+			Just key -> do
+				showStart message (show n)
+				next $ a key
