@@ -9,8 +9,15 @@ module Locations (
 	keyFile,
 	fileKey,
 	keyPaths,
+	keyPath,
 	gitAnnexLocation,
+	gitAnnexLink,
+	gitAnnexMapping,
+	gitAnnexInodeCache,
+	gitAnnexInodeSentinal,
+	gitAnnexInodeSentinalCache,
 	annexLocations,
+	annexLocation,
 	gitAnnexDir,
 	gitAnnexObjectDir,
 	gitAnnexTmpDir,
@@ -21,6 +28,7 @@ module Locations (
 	gitAnnexFsckState,
 	gitAnnexTransferDir,
 	gitAnnexCredsDir,
+	gitAnnexMergeDir,
 	gitAnnexJournalDir,
 	gitAnnexJournalLock,
 	gitAnnexIndex,
@@ -28,6 +36,7 @@ module Locations (
 	gitAnnexPidFile,
 	gitAnnexDaemonStatusFile,
 	gitAnnexLogFile,
+	gitAnnexFuzzTestLogFile,
 	gitAnnexHtmlShim,
 	gitAnnexUrlFile,
 	gitAnnexTmpCfgFile,
@@ -81,29 +90,68 @@ annexLocations key = map (annexLocation key) annexHashes
 annexLocation :: Key -> Hasher -> FilePath
 annexLocation key hasher = objectDir </> keyPath key hasher
 
-{- Annexed file's absolute location in a repository.
+{- Annexed object's absolute location in a repository.
  -
  - When there are multiple possible locations, returns the one where the
  - file is actually present.
  -
  - When the file is not present, returns the location where the file should
  - be stored.
+ -
+ - This does not take direct mode into account, so in direct mode it is not
+ - the actual location of the file's content.
  -}
-gitAnnexLocation :: Key -> Git.Repo -> IO FilePath
-gitAnnexLocation key r
-	| Git.repoIsLocalBare r =
-		{- Bare repositories default to hashDirLower for new
-		 - content, as it's more portable. -}
+gitAnnexLocation :: Key -> Git.Repo -> GitConfig -> IO FilePath
+gitAnnexLocation key r config = gitAnnexLocation' key r (annexCrippledFileSystem config)
+gitAnnexLocation' :: Key -> Git.Repo -> Bool -> IO FilePath
+gitAnnexLocation' key r crippled
+	{- Bare repositories default to hashDirLower for new
+	 - content, as it's more portable.
+	 -
+	 - Repositories on filesystems that are crippled also use
+	 - hashDirLower, since they do not use symlinks and it's
+	 - more portable. -}
+	| Git.repoIsLocalBare r || crippled =
 		check $ map inrepo $ annexLocations key
-	| otherwise =
-		{- Non-bare repositories only use hashDirMixed, so
-		 - don't need to do any work to check if the file is
-		 - present. -}
-		return $ inrepo $ annexLocation key hashDirMixed
-	where
-		inrepo d = Git.localGitDir r </> d
-		check locs@(l:_) = fromMaybe l <$> firstM doesFileExist locs
-		check [] = error "internal"
+	{- Non-bare repositories only use hashDirMixed, so
+	 - don't need to do any work to check if the file is
+	 - present. -}
+	| otherwise = return $ inrepo $ annexLocation key hashDirMixed
+  where
+	inrepo d = Git.localGitDir r </> d
+	check locs@(l:_) = fromMaybe l <$> firstM doesFileExist locs
+	check [] = error "internal"
+
+{- Calculates a symlink to link a file to an annexed object. -}
+gitAnnexLink :: FilePath -> Key -> Git.Repo -> IO FilePath
+gitAnnexLink file key r = do
+	cwd <- getCurrentDirectory
+	let absfile = fromMaybe whoops $ absNormPath cwd file
+	loc <- gitAnnexLocation' key r False
+	return $ relPathDirToFile (parentDir absfile) loc
+  where
+  	whoops = error $ "unable to normalize " ++ file
+
+{- File that maps from a key to the file(s) in the git repository.
+ - Used in direct mode. -}
+gitAnnexMapping :: Key -> Git.Repo -> GitConfig -> IO FilePath
+gitAnnexMapping key r config = do
+	loc <- gitAnnexLocation key r config
+	return $ loc ++ ".map"
+
+{- File that caches information about a key's content, used to determine
+ - if a file has changed.
+ - Used in direct mode. -}
+gitAnnexInodeCache :: Key -> Git.Repo -> GitConfig -> IO FilePath
+gitAnnexInodeCache key r config  = do
+	loc <- gitAnnexLocation key r config
+	return $ loc ++ ".cache"
+
+gitAnnexInodeSentinal :: Git.Repo -> FilePath
+gitAnnexInodeSentinal r = gitAnnexDir r </> "sentinal"
+
+gitAnnexInodeSentinalCache :: Git.Repo -> FilePath
+gitAnnexInodeSentinalCache r = gitAnnexInodeSentinal r ++ ".cache"
 
 {- The annex directory of a repository. -}
 gitAnnexDir :: Git.Repo -> FilePath
@@ -117,7 +165,7 @@ gitAnnexObjectDir r = addTrailingPathSeparator $ Git.localGitDir r </> objectDir
 gitAnnexTmpDir :: Git.Repo -> FilePath
 gitAnnexTmpDir r = addTrailingPathSeparator $ gitAnnexDir r </> "tmp"
 
-{- The temp file to use for a given key. -}
+{- The temp file to use for a given key's content. -}
 gitAnnexTmpLocation :: Key -> Git.Repo -> FilePath
 gitAnnexTmpLocation key r = gitAnnexTmpDir r </> keyFile key
 
@@ -142,6 +190,10 @@ gitAnnexFsckState r = gitAnnexDir r </> "fsckstate"
 gitAnnexCredsDir :: Git.Repo -> FilePath
 gitAnnexCredsDir r = addTrailingPathSeparator $ gitAnnexDir r </> "creds"
 
+{- .git/annex/merge/ is used for direct mode merges. -}
+gitAnnexMergeDir :: Git.Repo -> FilePath
+gitAnnexMergeDir r = addTrailingPathSeparator $ gitAnnexDir r </> "merge"
+
 {- .git/annex/transfer/ is used to record keys currently
  - being transferred, and other transfer bookkeeping info. -}
 gitAnnexTransferDir :: Git.Repo -> FilePath
@@ -176,6 +228,10 @@ gitAnnexDaemonStatusFile r = gitAnnexDir r </> "daemon.status"
 gitAnnexLogFile :: Git.Repo -> FilePath
 gitAnnexLogFile r = gitAnnexDir r </> "daemon.log"
 
+{- Log file for fuzz test. -}
+gitAnnexFuzzTestLogFile :: Git.Repo -> FilePath
+gitAnnexFuzzTestLogFile r = gitAnnexDir r </> "fuzztest.log"
+
 {- Html shim file used to launch the webapp. -}
 gitAnnexHtmlShim :: Git.Repo -> FilePath
 gitAnnexHtmlShim r = gitAnnexDir r </> "webapp.html"
@@ -201,11 +257,14 @@ gitAnnexRemotesDir r = addTrailingPathSeparator $ gitAnnexDir r </> "remotes"
 gitAnnexAssistantDefaultDir :: FilePath
 gitAnnexAssistantDefaultDir = "annex"
 
-{- Checks a symlink target to see if it appears to point to annexed content. -}
+{- Checks a symlink target to see if it appears to point to annexed content.
+ -
+ - We only look at paths inside the .git directory, and not at the .git
+ - directory itself, because GIT_DIR may cause a directory name other
+ - than .git to be used.
+ -}
 isLinkToAnnex :: FilePath -> Bool
-isLinkToAnnex s = ('/':d) `isInfixOf` s || d `isPrefixOf` s
-	where
-		d = ".git" </> objectDir
+isLinkToAnnex s = (pathSeparator:objectDir) `isInfixOf` s
 
 {- Converts a key into a filename fragment without any directory.
  -
@@ -232,8 +291,8 @@ keyFile key = replace "/" "%" $ replace ":" "&c" $
  -}
 keyPath :: Key -> Hasher -> FilePath
 keyPath key hasher = hasher key </> f </> f
-	where
-		f = keyFile key
+  where
+	f = keyFile key
 
 {- All possibile locations to store a key using different directory hashes. -}
 keyPaths :: Key -> [FilePath]
@@ -249,7 +308,8 @@ fileKey file = file2key $
 {- for quickcheck -}
 prop_idempotent_fileKey :: String -> Bool
 prop_idempotent_fileKey s = Just k == fileKey (keyFile k)
-	where k = stubKey { keyName = s, keyBackendName = "test" }
+  where
+	k = stubKey { keyName = s, keyBackendName = "test" }
 
 {- Two different directory hashes may be used. The mixed case hash
  - came first, and is fine, except for the problem of case-strict
@@ -262,14 +322,14 @@ annexHashes = [hashDirLower, hashDirMixed]
 
 hashDirMixed :: Hasher
 hashDirMixed k = addTrailingPathSeparator $ take 2 dir </> drop 2 dir
-	where
-		dir = take 4 $ display_32bits_as_dir =<< [a,b,c,d]
-		ABCD (a,b,c,d) = md5 $ md5FilePath $ key2file k
+  where
+	dir = take 4 $ display_32bits_as_dir =<< [a,b,c,d]
+	ABCD (a,b,c,d) = md5 $ md5FilePath $ key2file k
 
 hashDirLower :: Hasher
 hashDirLower k = addTrailingPathSeparator $ take 3 dir </> drop 3 dir
-	where
-		dir = take 6 $ md5s $ md5FilePath $ key2file k
+  where
+	dir = take 6 $ md5s $ md5FilePath $ key2file k
 
 {- modified version of display_32bits_as_hex from Data.Hash.MD5
  -   Copyright (C) 2001 Ian Lynagh 
@@ -277,13 +337,13 @@ hashDirLower k = addTrailingPathSeparator $ take 3 dir </> drop 3 dir
  -}
 display_32bits_as_dir :: Word32 -> String
 display_32bits_as_dir w = trim $ swap_pairs cs
-	where 
-		-- Need 32 characters to use. To avoid inaverdently making
-		-- a real word, use letters that appear less frequently.
-		chars = ['0'..'9'] ++ "zqjxkmvwgpfZQJXKMVWGPF"
-		cs = map (\x -> getc $ (shiftR w (6*x)) .&. 31) [0..7]
-		getc n = chars !! fromIntegral n
-		swap_pairs (x1:x2:xs) = x2:x1:swap_pairs xs
-		swap_pairs _ = []
-		-- Last 2 will always be 00, so omit.
-		trim = take 6
+  where 
+	-- Need 32 characters to use. To avoid inaverdently making
+	-- a real word, use letters that appear less frequently.
+	chars = ['0'..'9'] ++ "zqjxkmvwgpfZQJXKMVWGPF"
+	cs = map (\x -> getc $ (shiftR w (6*x)) .&. 31) [0..7]
+	getc n = chars !! fromIntegral n
+	swap_pairs (x1:x2:xs) = x2:x1:swap_pairs xs
+	swap_pairs _ = []
+	-- Last 2 will always be 00, so omit.
+	trim = take 6
