@@ -6,12 +6,18 @@
  -}
 
 module Logs.Trust (
+	trustLog,
 	TrustLevel(..),
 	trustGet,
+	trustMap,
 	trustSet,
 	trustPartition,
+	trustExclude,
 	lookupTrust,
+	trustMapLoad,
 	trustMapRaw,
+	
+	prop_parse_show_TrustLog,
 ) where
 
 import qualified Data.Map as M
@@ -23,7 +29,6 @@ import qualified Annex.Branch
 import qualified Annex
 import Logs.UUIDBased
 import Remote.List
-import Config
 import qualified Types.Remote
 
 {- Filename of trust.log. -}
@@ -65,27 +70,32 @@ trustPartition level ls
 		candidates <- trustGet level
 		return $ partition (`elem` candidates) ls
 
-{- Read the trustLog into a map, overriding with any
- - values from forcetrust or the git config. The map is cached for speed. -}
-trustMap :: Annex TrustMap
-trustMap = do
-	cached <- Annex.getState Annex.trustmap
-	case cached of
-		Just m -> return m
-		Nothing -> do
-			overrides <- Annex.getState Annex.forcetrust
-			logged <- trustMapRaw
-			configured <- M.fromList . catMaybes
-				<$> (mapM configuredtrust =<< remoteList)
-			let m = M.union overrides $ M.union configured logged
-			Annex.changeState $ \s -> s { Annex.trustmap = Just m }
-			return m
-	where
-		configuredtrust r =
-			maybe Nothing (\l -> Just (Types.Remote.uuid r, l)) <$>
-				maybe Nothing readTrustLevel
-					<$> getTrustLevel (Types.Remote.repo r)
+{- Filters UUIDs to those not matching a TrustLevel. -}
+trustExclude :: TrustLevel -> [UUID] -> Annex [UUID]
+trustExclude level ls = snd <$> trustPartition level ls
 
+{- trustLog in a map, overridden with any values from forcetrust or
+ - the git config. The map is cached for speed. -}
+trustMap :: Annex TrustMap
+trustMap = maybe trustMapLoad return =<< Annex.getState Annex.trustmap
+
+{- Loads the map, updating the cache, -}
+trustMapLoad :: Annex TrustMap
+trustMapLoad = do
+	overrides <- Annex.getState Annex.forcetrust
+	logged <- trustMapRaw
+	configured <- M.fromList . catMaybes
+		<$> (map configuredtrust <$> remoteList)
+	let m = M.union overrides $ M.union configured logged
+	Annex.changeState $ \s -> s { Annex.trustmap = Just m }
+	return m
+  where
+	configuredtrust r = (\l -> Just (Types.Remote.uuid r, l))
+		=<< readTrustLevel 
+		=<< remoteAnnexTrustLevel (Types.Remote.gitconfig r)
+
+{- Does not include forcetrust or git config values, just those from the
+ - log file. -}
 trustMapRaw :: Annex TrustMap
 trustMapRaw = simpleMap . parseLog (Just . parseTrustLog)
 	<$> Annex.Branch.get trustLog
@@ -94,14 +104,19 @@ trustMapRaw = simpleMap . parseLog (Just . parseTrustLog)
  - trust status, which is why this defaults to Trusted. -}
 parseTrustLog :: String -> TrustLevel
 parseTrustLog s = maybe Trusted parse $ headMaybe $ words s
-	where
-		parse "1" = Trusted
-		parse "0" = UnTrusted
-		parse "X" = DeadTrusted
-		parse _ = SemiTrusted
+  where
+	parse "1" = Trusted
+	parse "0" = UnTrusted
+	parse "X" = DeadTrusted
+	parse _ = SemiTrusted
 
 showTrustLog :: TrustLevel -> String
 showTrustLog Trusted = "1"
 showTrustLog UnTrusted = "0"
 showTrustLog DeadTrusted = "X"
 showTrustLog SemiTrusted = "?"
+
+prop_parse_show_TrustLog :: Bool
+prop_parse_show_TrustLog = all check [minBound .. maxBound]
+  where
+	check l = parseTrustLog (showTrustLog l) == l
